@@ -23,6 +23,14 @@
 ".color-%1$d {"\
 "  background-color: %2$s"\
 "}"\
+".color-%1$d.horizontal.timed:dir(ltr) {"\
+"  background-color: transparent;"\
+"  border-left: solid 4px %2$s;"\
+"}"\
+".color-%1$d.horizontal.timed:dir(rtl) {"\
+"  background-color: transparent;"\
+"  border-right: solid 4px %2$s;"\
+"}"\
 ".color-%1$d.slanted {"\
 "  background-color: transparent;"\
 "  background-image: linear-gradient(100deg, /* left edge */"\
@@ -257,7 +265,12 @@
 "                                    alpha(%2$s, 0) 10px,"\
 "                                    alpha(%2$s, 0) 16px,"\
 "                                    %2$s	   17px);"\
-"}"
+"}"\
+".color-%1$d.slanted box { margin: 0 16px 0 16px; }"\
+".color-%1$d.slanted-start box { margin-left: 16px; }"\
+".color-%1$d.slanted-start:dir(rtl) box { margin-right: 16px; }"\
+".color-%1$d.slanted-end box { margin-right: 16px; }"\
+".color-%1$d.slanted-end:dir(rtl) box { margin-left: 16px; }"
 
 #endif /* CSS_CODE_H */
 
