@@ -722,9 +722,9 @@ show_new_event_widget (GcalView   *view,
   window->event_creation_data->x = x;
   window->event_creation_data->y = y;
   window->event_creation_data->start_date = g_date_time_ref (start_span);
-  if (end_span != NULL)
-    window->event_creation_data->end_date = g_date_time_ref (end_span);
-  g_debug ("[show_new_event] position (%f, %f)", x, y);
+  window->event_creation_data->end_date = end_span ? g_date_time_ref (end_span) : NULL;
+
+  g_debug ("Quick add popover position (%f, %f)", x, y);
 
   /* Setup the quick add popover's dates */
   gcal_quick_add_popover_set_date_start (GCAL_QUICK_ADD_POPOVER (window->quick_add_popover), start_span);
@@ -1457,11 +1457,19 @@ gcal_window_state_event (GtkWidget           *widget,
 }
 
 static void
-gcal_window_class_init(GcalWindowClass *klass)
+gcal_window_class_init (GcalWindowClass *klass)
 {
   GObjectClass *object_class;
   GtkWidgetClass *widget_class;
 
+  g_type_ensure (GCAL_TYPE_EDIT_DIALOG);
+  g_type_ensure (GCAL_TYPE_MONTH_VIEW);
+  g_type_ensure (GCAL_TYPE_QUICK_ADD_POPOVER);
+  g_type_ensure (GCAL_TYPE_SEARCH_VIEW);
+  g_type_ensure (GCAL_TYPE_SOURCE_DIALOG);
+  g_type_ensure (GCAL_TYPE_WEEK_VIEW);
+  g_type_ensure (GCAL_TYPE_YEAR_VIEW);
+
   object_class = G_OBJECT_CLASS (klass);
   object_class->finalize = gcal_window_finalize;
   object_class->set_property = gcal_window_set_property;
