@@ -19,6 +19,7 @@
 
 #define G_LOG_DOMAIN "GcalWeekGrid"
 
+#include "gcal-debug.h"
 #include "gcal-week-grid.h"
 #include "gcal-week-view.h"
 #include "gcal-utils.h"
@@ -614,8 +615,8 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
   gboolean ltr;
   gdouble minutes_height;
   gdouble column_width;
+  guint x, y;
   guint i;
-  guint x;
 
   /* No need to relayout stuff if nothing changed */
   if (allocation->height == gtk_widget_get_allocated_height (widget) &&
@@ -697,15 +698,21 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
           width = column_width / events_at_range - margin.left - margin.right;
           height = (data->end - data->start) * minutes_height - margin.top - margin.bottom;
           offset = (width + margin.left + margin.right) * widget_index;
+          y = (data->start % MINUTES_PER_DAY) * minutes_height + margin.top;
 
           if (ltr)
             x = column_width * i + offset + allocation->x + margin.left + 1;
           else
             x = allocation->width - width - (column_width * i + offset + allocation->x + margin.left + 1);
 
+          /* TODO: find a better way to handle line widths */
+          height -= 2;
+          width -= 1;
+          y += 2;
+
           /* Setup the child position and size */
           child_allocation.x = x;
-          child_allocation.y = (data->start % MINUTES_PER_DAY) * minutes_height + margin.top;
+          child_allocation.y = y;
           child_allocation.width = width;
           child_allocation.height = height;
 
@@ -980,12 +987,12 @@ gcal_week_grid_drag_drop (GtkWidget      *widget,
                           gint            y,
                           guint           time)
 {
+  GcalRecurrenceModType mod;
   GcalWeekGrid *self;
   g_autoptr (GDateTime) week_start;
   g_autoptr (GDateTime) dnd_date;
   g_autoptr (GDateTime) new_end;
   GtkWidget *event_widget;
-  ECalObjModType mod;
   GcalEvent *event;
   ESource *source;
   GTimeSpan timespan = 0;
@@ -997,7 +1004,7 @@ gcal_week_grid_drag_drop (GtkWidget      *widget,
   drop_cell = get_dnd_cell (widget, x, y);
   event_widget = gtk_drag_get_source_widget (context);
 
-  mod = E_CAL_OBJ_MOD_THIS;
+  mod = GCAL_RECURRENCE_MOD_THIS_ONLY;
   week_start = NULL;
   dnd_date = NULL;
   new_end = NULL;
@@ -1238,32 +1245,33 @@ gcal_week_grid_remove_event (GcalWeekGrid *self,
 }
 
 GList*
-gcal_week_grid_get_children_by_uuid (GcalWeekGrid *self,
-                                     const gchar  *uid)
+gcal_week_grid_get_children_by_uuid (GcalWeekGrid          *self,
+                                     GcalRecurrenceModType  mod,
+                                     const gchar           *uid)
 {
   GPtrArray *widgets;
+  GList *events;
   GList *result;
   guint i;
 
+  GCAL_ENTRY;
+
+  events = NULL;
   result = NULL;
   widgets = gcal_range_tree_get_data_at_range (self->events, 0, MAX_MINUTES);
 
   for (i = 0; widgets && i < widgets->len; i++)
     {
-      ChildData *data;
-      GcalEvent *event;
-
-      data = g_ptr_array_index (widgets, i);
-      event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (data->widget));
-
-      if (g_strcmp0 (gcal_event_get_uid (event), uid) == 0)
-        result = g_list_prepend (result, data->widget);
-
+      ChildData *data = g_ptr_array_index (widgets, i);
+      events = g_list_prepend (events, data->widget);
     }
 
-  g_clear_pointer (&widgets, g_ptr_array_unref);
+  result = filter_event_list_by_uid_and_modtype (events, mod, uid);
 
-  return result;
+  g_clear_pointer (&widgets, g_ptr_array_unref);
+  g_clear_pointer (&events, g_list_free);
+
+  GCAL_RETURN (result);
 }
 
 void
