@@ -120,6 +120,7 @@ struct _GcalYearView
 enum {
   PROP_0,
   PROP_DATE,
+  PROP_MANAGER,
   PROP_SHOW_WEEK_NUMBERS,
   LAST_PROP
 };
@@ -474,7 +475,7 @@ update_sidebar_headers (GtkListBoxRow *row,
   year_view = GCAL_YEAR_VIEW (user_data);
   row_child = gtk_bin_get_child (GTK_BIN (row));
 
-  if (row_child == NULL)
+  if (!row_child)
     return;
 
   row_event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (row_child));
@@ -482,7 +483,7 @@ update_sidebar_headers (GtkListBoxRow *row,
   row_date = g_date_time_to_local (row_date);
   row_shift = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (row_child), "shift"));
 
-  if (before != NULL)
+  if (before)
     {
       before_child = gtk_bin_get_child (GTK_BIN (before));
       before_event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (before_child));
@@ -519,34 +520,6 @@ update_sidebar_headers (GtkListBoxRow *row,
       gtk_container_add (GTK_CONTAINER (row_header), label);
     }
 
-  if (!gcal_event_is_multiday (row_event) &&
-      !gcal_event_get_all_day (row_event) &&
-      (before_date == NULL || g_date_time_get_hour (before_date) != g_date_time_get_hour (row_date)))
-    {
-      gchar *time;
-      GtkWidget *label;
-      gint hour;
-
-      hour = g_date_time_get_hour (row_date);
-
-      if (year_view->use_24h_format)
-        time = g_strdup_printf ("%.2d:00", hour);
-      else
-        time = g_strdup_printf ("%.2d:00 %s", hour % 12, hour < 12 ? "AM" : "PM");
-
-      label = gtk_label_new (time);
-      gtk_style_context_add_class (gtk_widget_get_style_context (label), GTK_STYLE_CLASS_DIM_LABEL);
-      g_object_set (label, "margin", 6, "halign", GTK_ALIGN_START, NULL);
-
-      if (row_header == NULL)
-        row_header = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-
-      gtk_container_add (GTK_CONTAINER (row_header), label);
-      gtk_container_add (GTK_CONTAINER (row_header), gtk_separator_new (GTK_ORIENTATION_HORIZONTAL));
-
-      g_free (time);
-    }
-
   if (row_header != NULL)
     gtk_widget_show_all (row_header);
   gtk_list_box_row_set_header (row, row_header);
@@ -754,8 +727,9 @@ draw_month_grid (GcalYearView *year_view,
   GdkRGBA color;
   gint layout_width, layout_height, i, j, sw;
   gint column, row;
+  gboolean column_is_workday;
   gdouble x, y, box_side, box_padding_top, box_padding_start;
-  gint days_delay, days, shown_rows, sunday_idx;
+  gint days_delay, days, shown_rows;
   gchar *str, *nr_day, *nr_week;
   gboolean selected_day;
   icaltimetype today;
@@ -824,7 +798,6 @@ draw_month_grid (GcalYearView *year_view,
   days_delay = (time_day_of_week (1, month_nr, year_view->date->year) - year_view->first_weekday + 7) % 7;
   days = days_delay + icaltime_days_in_month (month_nr + 1, year_view->date->year);
   shown_rows = ceil (days / 7.0);
-  sunday_idx = year_view->k * 7 + sw * ((7 - year_view->first_weekday) % 7) + year_view->show_week_numbers * (1 - year_view->k);
 
   today = icaltime_today ();
   today.year = year_view->date->year;
@@ -838,6 +811,7 @@ draw_month_grid (GcalYearView *year_view,
   for (i = 0; i < 7 * shown_rows; i++)
     {
       column = (i % 7) + (year_view->k || year_view->show_week_numbers);
+      column_is_workday = is_workday ((7 * year_view->k + sw * (column - (year_view->show_week_numbers * !year_view->k) + (sw * year_view->first_weekday))) % 7);
       row = i / 7;
 
       j = 7 * ((i + 7 * year_view->k) / 7) + sw * (i % 7) + (1 - year_view->k);
@@ -966,10 +940,10 @@ draw_month_grid (GcalYearView *year_view,
 
           gtk_style_context_set_state (context, state_flags);
         }
-      else if (column == sunday_idx)
+      else if (!column_is_workday)
         {
           gtk_style_context_save (context);
-          gtk_style_context_add_class (context, "sunday");
+          gtk_style_context_add_class (context, "non-workday");
           gtk_render_layout (context, cr,
                              box_side * column + x + sw * box_padding_start - year_view->k * layout_width,
                              box_side * (row + 1) + y + box_padding_top,
@@ -994,8 +968,8 @@ draw_month_grid (GcalYearView *year_view,
 
           if (selected_day)
             gtk_style_context_add_class (context, "with-events-selected");
-          else if (column == sunday_idx)
-            gtk_style_context_add_class (context, "with-events-sunday");
+          else if (!column_is_workday)
+            gtk_style_context_add_class (context, "with-events-non-workday");
 
           box_padding_start = MAX (0, (box_side - VISUAL_CLUES_SIDE) / 2);
           gtk_render_background (context, cr,
@@ -1091,7 +1065,7 @@ draw_navigator (GcalYearView *year_view,
 
   /* read header from CSS code related to the view */
   gtk_style_context_save (context);
-  gtk_style_context_add_class (context, "start-header");
+  gtk_style_context_add_class (context, "primary-label");
 
   header_str = g_strdup_printf ("%d", year_view->date->year);
   gtk_style_context_get (context, state_flags, "font", &font_desc, NULL);
@@ -1374,7 +1348,7 @@ calculate_sizes (GcalYearView *self)
   /* get header info from CSS */
   context = gtk_widget_get_style_context (GTK_WIDGET (self));
   gtk_style_context_save (context);
-  gtk_style_context_add_class (context, "start-header");
+  gtk_style_context_add_class (context, "primary-label");
   gtk_style_context_get (context, gtk_style_context_get_state (context),
                          "padding-top", &padding_top,
                          "padding-bottom", &padding_bottom,
@@ -1487,8 +1461,8 @@ navigator_drag_drop_cb (GcalYearView   *self,
 
       if (!is_title)
         {
+          GcalRecurrenceModType mod;
           GcalEventWidget *event_widget;
-          ECalObjModType mod;
           GcalEvent *event;
           GDateTime *start_dt, *end_dt;
           GDateTime *drop_date;
@@ -1497,7 +1471,7 @@ navigator_drag_drop_cb (GcalYearView   *self,
           event_widget = GCAL_EVENT_WIDGET (gtk_drag_get_source_widget (context));
           event = gcal_event_widget_get_event (event_widget);
           source = gcal_event_get_source (event);
-          mod = E_CAL_OBJ_MOD_THIS;
+          mod = GCAL_RECURRENCE_MOD_THIS_ONLY;
 
           if (gcal_event_has_recurrence (event) &&
               !ask_recurrence_modification_type (GTK_WIDGET (self), &mod, source))
@@ -1588,26 +1562,17 @@ gcal_year_view_set_date (GcalView     *view,
 }
 
 static GList*
-gcal_year_view_get_children_by_uuid (GcalView    *view,
-                                     const gchar *uuid)
+gcal_year_view_get_children_by_uuid (GcalView              *view,
+                                     GcalRecurrenceModType  mod,
+                                     const gchar           *uuid)
 {
-  GcalYearView *year_view = GCAL_YEAR_VIEW (view);
-  GList *children, *l, *result = NULL;
+  GcalYearView *self = GCAL_YEAR_VIEW (view);
+  GList *result, *children;
 
-  children = gtk_container_get_children (GTK_CONTAINER (year_view->events_sidebar));
+  result = NULL;
 
-  for (l = children; l != NULL; l = g_list_next (l))
-    {
-      GcalEventWidget *child_widget;
-      GcalEvent *event;
-
-      child_widget = GCAL_EVENT_WIDGET (gtk_bin_get_child (GTK_BIN (l->data)));
-      event = gcal_event_widget_get_event (child_widget);
-
-
-      if (child_widget != NULL && g_strcmp0 (uuid, gcal_event_get_uid (event)) == 0)
-        result = g_list_append (result, child_widget);
-    }
+  children = gtk_container_get_children (GTK_CONTAINER (self->sidebar));
+  result = filter_event_list_by_uid_and_modtype (children, mod, uuid);
 
   g_list_free (children);
 
@@ -1659,6 +1624,10 @@ gcal_year_view_get_property (GObject    *object,
       g_value_set_boxed (value, self->date);
       break;
 
+    case PROP_MANAGER:
+      g_value_set_object (value, self->manager);
+      break;
+
     case PROP_SHOW_WEEK_NUMBERS:
       g_value_set_boolean (value, self->show_week_numbers);
       break;
@@ -1682,6 +1651,17 @@ gcal_year_view_set_property (GObject      *object,
       gcal_view_set_date (GCAL_VIEW (self), g_value_get_boxed (value));
       break;
 
+    case PROP_MANAGER:
+      self->manager = g_value_dup_object (value);
+
+      g_signal_connect_swapped (gcal_manager_get_clock (self->manager),
+                                "day-changed",
+                                G_CALLBACK (gtk_widget_queue_draw),
+                                self->navigator);
+
+      g_object_notify (object, "manager");
+      break;
+
     case PROP_SHOW_WEEK_NUMBERS:
       if (self->show_week_numbers != g_value_get_boolean (value))
         {
@@ -1834,7 +1814,7 @@ gcal_year_view_component_added (ECalDataModelSubscriber *subscriber,
 
   /* Add the event to the cache */
   for (i = start_month; i <= end_month; i++)
-    g_ptr_array_add (self->events[i], event);
+    g_ptr_array_add (self->events[i], g_object_ref (event));
 
   update_sidebar (self);
 
@@ -1956,6 +1936,7 @@ gcal_year_view_class_init (GcalYearViewClass *klass)
   widget_class->direction_changed = gcal_year_view_direction_changed;
 
   g_object_class_override_property (object_class, PROP_DATE, "active-date");
+  g_object_class_override_property (object_class, PROP_MANAGER, "manager");
 
   g_object_class_install_property (object_class,
                                    PROP_SHOW_WEEK_NUMBERS,
@@ -1966,10 +1947,13 @@ gcal_year_view_class_init (GcalYearViewClass *klass)
                                                          G_PARAM_READWRITE));
 
   /* FIXME: it will problably go back to GcalView */
-  signals[EVENT_ACTIVATED] = g_signal_new ("event-activated", GCAL_TYPE_YEAR_VIEW, G_SIGNAL_RUN_LAST,
-                                           0,
-                                           NULL, NULL, NULL,
-                                           G_TYPE_NONE, 1, GCAL_TYPE_EVENT_WIDGET);
+  signals[EVENT_ACTIVATED] = g_signal_new ("event-activated",
+                                           GCAL_TYPE_YEAR_VIEW,
+                                           G_SIGNAL_RUN_LAST,
+                                           0, NULL, NULL, NULL,
+                                           G_TYPE_NONE,
+                                           1,
+                                           GCAL_TYPE_EVENT_WIDGET);
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/year-view.ui");
 
@@ -2003,7 +1987,7 @@ gcal_year_view_init (GcalYearView *self)
   guint i;
 
   for (i = 0; i < 12; i++)
-    self->events[i] = g_ptr_array_new ();
+    self->events[i] = g_ptr_array_new_with_free_func (g_object_unref);
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
@@ -2051,18 +2035,6 @@ gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *ifa
 }
 
 /* Public API */
-void
-gcal_year_view_set_manager (GcalYearView *year_view,
-                            GcalManager  *manager)
-{
-  year_view->manager = manager;
-
-  g_signal_connect_swapped (gcal_manager_get_clock (manager),
-                            "day-changed",
-                            G_CALLBACK (gtk_widget_queue_draw),
-                            year_view->navigator);
-}
-
 void
 gcal_year_view_set_first_weekday (GcalYearView *year_view,
                                   gint          nr_day)
